package com.github.zengfr.easymodbus4j.cache;

import com.github.zengfr.easymodbus4j.ModbusConfs;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Set;
import java.util.concurrent.TimeUnit;

public class ModebusFrameCache {
    private static final Logger logger = LoggerFactory.getLogger(ModebusFrameCache.class);

    private final Cache<Integer, ModbusFrame> cache = CacheBuilder.newBuilder()
            .expireAfterWrite(ModbusConfs.FRAME_CACHE_EXPIRE, TimeUnit.MILLISECONDS).build();

    public ModbusFrame get(Integer key) {
        //logger.debug("get:{}", key);
        return this.cache.getIfPresent(key);
    }

    public void put(ModbusFrame value) {
        put(value.getHeader().getTransactionIdentifier(), value);
    }

    public void put(Integer key, ModbusFrame value) {
        this.cache.put(key, value);
        //ModbusFrame modbusFrame = get(key);
        //logger.info("====={}", modbusFrame);
        //logger.debug("put:{},{}", key, Short.valueOf(value.getFunction().getFunctionCode()));
    }

    public Set<Integer> keySet() {
        return this.cache.asMap().keySet();
    }

    public long size() {
        return this.cache.size();
    }

    public void remove(Integer key) {
        this.cache.invalidate(key);
        //logger.debug("remove:{}", key);
    }
}
