package com.github.zengfr.easymodbus4j.cache;

import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.logging.InternalLoggerFactory;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class ModebusFrameCacheFactory {
    private static class ModebusFrameCacheFactoryHolder {
        private static final ModebusFrameCacheFactory INSTANCE = new ModebusFrameCacheFactory();
    }

    protected static final InternalLogger logger = InternalLoggerFactory.getInstance(ModebusFrameCacheFactory.class);

    public static ModebusFrameCacheFactory getInstance() {
        return ModebusFrameCacheFactoryHolder.INSTANCE;
    }

    private final Map<String, ModebusFrameCache> cacheMap = new ConcurrentHashMap<>();

    protected ModebusFrameCache getCache(String key) {
        if (!this.cacheMap.containsKey(key))
            synchronized (this.cacheMap) {
                if (!this.cacheMap.containsKey(key))
                    this.cacheMap.put(key, new ModebusFrameCache());
            }
        return this.cacheMap.get(key);
    }

    public ModebusFrameCache getRequestCache(Short funcCode) {
        return getCache("req" + funcCode);
    }

    public ModebusFrameCache getResponseCache(Short funcCode) {
        return getCache("resp" + funcCode);
    }
}
