package com.github.zengfr.easymodbus4j.codec.tcp;

import com.github.zengfr.easymodbus4j.codec.util.ModbusFunctionDecoderUtil;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusHeader;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.logging.InternalLoggerFactory;

import java.util.List;

public class ModbusTcpDecoder extends ByteToMessageDecoder {
    private static final InternalLogger logger = InternalLoggerFactory.getInstance(ModbusTcpDecoder.class);

    private final boolean isSlave;

    public ModbusTcpDecoder(boolean isSlave) {
        this.isSlave = isSlave;
    }

    protected void decode(ChannelHandlerContext ctx, ByteBuf buffer, List<Object> out) {
        logger.debug("decode");
        if (buffer.capacity() < 8)
            return;
        buffer.markReaderIndex();
        ModbusFrame frame = decodeFrame(buffer, this.isSlave);
        if (frame != null) {
            out.add(frame);
        } else {
            buffer.resetReaderIndex();
            ctx.fireChannelRead(buffer);
        }
    }

    public static ModbusFrame decodeFrame(ByteBuf buffer, boolean isSlave) {
        if (buffer.capacity() < 8)
            return null;
        ModbusHeader mbapHeader = ModbusHeader.decode(buffer);
        ModbusFunction function = decodeFunction(buffer, isSlave);
        ModbusFrame frame = new ModbusFrame(mbapHeader, function);
        return frame;
    }

    public static ModbusFunction decodeFunction(ByteBuf buffer, boolean isRequest) {
        ModbusFunction function = null;
        short functionCode = buffer.readUnsignedByte();
        if (isRequest) {
            function = ModbusFunctionDecoderUtil.decodeReqFunction(functionCode);
        } else {
            function = ModbusFunctionDecoderUtil.decodeRespFunction(functionCode);
        }
        function.decode(buffer.readBytes(buffer.readableBytes()));
        return function;
    }
}
