package com.github.zengfr.easymodbus4j.codec.util;

import com.github.zengfr.easymodbus4j.func.request.*;
import com.github.zengfr.easymodbus4j.func.response.*;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.util.ModbusFunctionUtil;

public class ModbusFunctionDecoderUtil {

    public static ModbusFunction decodeReqFunction(short functionCode) {
        ModbusFunction function = null;
        switch (functionCode) {
            case 1:
                function = new ReadCoilsRequest();
                break;
            case 2:
                function = new ReadDiscreteInputsRequest();
                break;
            case 4:
                function = new ReadInputRegistersRequest();
                break;
            case 3:
                function = new ReadHoldingRegistersRequest();
                break;
            case 5:
                function = new WriteSingleCoilRequest();
                break;
            case 6:
                function = new WriteSingleRegisterRequest();
                break;
            case 15:
                function = new WriteMultipleCoilsRequest();
                break;
            case 16:
                function = new WriteMultipleRegistersRequest();
                break;
        }
        return function;
    }

    public static ModbusFunction decodeRespFunction(short functionCode) {
        ModbusFunction function = null;
        switch (functionCode) {
            case 1:
                function = new ReadCoilsResponse();
                break;
            case 2:
                function = new ReadDiscreteInputsResponse();
                break;
            case 4:
                function = new ReadInputRegistersResponse();
                break;
            case 3:
                function = new ReadHoldingRegistersResponse();
                break;
            case 5:
                function = new WriteSingleCoilResponse();
                break;
            case 6:
                function = new WriteSingleRegisterResponse();
                break;
            case 15:
                function = new WriteMultipleCoilsResponse();
                break;
            case 16:
                function = new WriteMultipleRegistersResponse();
                break;
        }
        if (ModbusFunctionUtil.isError(functionCode)) {
            function = new ErrorFunctionResponse(functionCode);
        } else if (function == null) {
            function = new ErrorFunctionResponse(functionCode, (short) 1);
        }
        return function;
    }
}
