package com.github.zengfr.easymodbus4j.common.util;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.List;
import java.util.Random;

public class BitSetUtil {
    public static BitSet copy(BitSet b) {
        return (BitSet) b.clone();
    }

    public static BitSet getRandomBits(int n, Random random) {
        BitSet bits = new BitSet(n);
        for (int i = 0; i < n; i++)
            bits.set(i, random.nextBoolean());
        return bits;
    }

    public static String toString(BitSet bits, int n) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; i++)
            sb.append(bits.get(i) ? "1 " : "0 ");
        return sb.toString();
    }

    public static String toString(BitSet bits) {
        return toString(bits, bits.length());
    }

    public static boolean innerProduct(BitSet a, BitSet b) {
        BitSet buffer = copy(a);
        buffer.and(b);
        return isOdd(buffer.cardinality());
    }

    private static boolean isOdd(int n) {
        return (n % 2 != 0);
    }

    public static BitSet fromList(List<Boolean> list) {
        BitSet bitset = new BitSet(list.size());
        for (int i = 0; i < list.size(); i++)
            bitset.set(i, list.get(i).booleanValue());
        return bitset;
    }

    public static BitSet fromArray(boolean[] array) {
        BitSet bitset = new BitSet(array.length);
        for (int i = 0; i < array.length; i++)
            bitset.set(i, array[i]);
        return bitset;
    }

    public static boolean[] toArray(BitSet bitset, int length) {
        if (length < 0)
            throw new IllegalArgumentException("Size of array must not be negative but was + " + length);
        boolean[] array = new boolean[length];
        for (int i = 0; i < length; i++)
            array[i] = bitset.get(i);
        return array;
    }

    public static List<Boolean> toList(BitSet bitset, int n) {
        if (n < 0)
            throw new IllegalArgumentException("Size of list must not be negative but was + " + n);
        List<Boolean> list = new ArrayList<>(n);
        for (int i = 0; i < n; i++)
            list.add(Boolean.valueOf(bitset.get(i)));
        return list;
    }
}
