package com.github.zengfr.easymodbus4j.common.util;

import java.nio.*;

public class ByteUtil {
    public static byte[] toByteArrayInt(int[] values) {
        ByteBuffer bf = ByteBuffer.allocate(4 * values.length);
        for (int value : values)
            bf.putInt(value);
        return bf.array();
    }

    public static byte[] floatToByte(float[] input) {
        byte[] ret = new byte[input.length * 4];
        for (int x = 0; x < input.length; x++)
            ByteBuffer.wrap(ret, x * 4, 4).putFloat(input[x]);
        return ret;
    }

    public static float[] byteToFloat(byte[] input) {
        float[] ret = new float[input.length / 4];
        for (int x = 0; x < input.length; x += 4)
            ret[x / 4] = ByteBuffer.wrap(input, x, 4).getFloat();
        return ret;
    }

    public static float[] toFloatArray(byte[] bytes) {
        return toFloatArray(bytes, true);
    }

    public static float[] toFloatArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        FloatBuffer fb = buffer.asFloatBuffer();
        float[] floatArray = new float[fb.limit()];
        fb.get(floatArray);
        return floatArray;
    }

    public static double[] toDoubleArray(byte[] bytes) {
        return toDoubleArray(bytes, true);
    }

    public static double[] toDoubleArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        DoubleBuffer db = buffer.asDoubleBuffer();
        double[] doubleArray = new double[db.limit()];
        db.get(doubleArray);
        return doubleArray;
    }

    public static long[] toLongArray(byte[] bytes) {
        return toLongArray(bytes, true);
    }

    public static long[] toLongArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        LongBuffer lb = buffer.asLongBuffer();
        long[] longArray = new long[lb.limit()];
        lb.get(longArray);
        return longArray;
    }

    public static int[] toIntArray(byte[] bytes) {
        return toIntArray(bytes, true);
    }

    public static int[] toIntArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        IntBuffer ib = buffer.asIntBuffer();
        int[] intArray = new int[ib.limit()];
        ib.get(intArray);
        return intArray;
    }

    public static int[] toUShortArray(byte[] bytes) {
        return toUShortArray(bytes, true);
    }

    public static int[] toUShortArray(byte[] bytes, boolean bigEndian) {
        short[] shortArray = toShortArray(bytes, bigEndian);
        int[] ushortArray = new int[shortArray.length];
        for (int i = 0; i < shortArray.length; i++)
            ushortArray[i] = shortArray[i] & 0xFFFF;
        return ushortArray;
    }

    public static short[] toShortArray(byte[] bytes) {
        return toShortArray(bytes, true);
    }

    public static short[] toShortArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        ShortBuffer sb = buffer.asShortBuffer();
        short[] shortArray = new short[sb.limit()];
        sb.get(shortArray);
        return shortArray;
    }

    public static char[] toCharArray(byte[] bytes) {
        return toCharArray(bytes, true);
    }

    public static char[] toCharArray(byte[] bytes, boolean bigEndian) {
        ByteBuffer buffer = ByteBuffer.wrap(bytes).order(bigEndian ? ByteOrder.BIG_ENDIAN : ByteOrder.LITTLE_ENDIAN);
        CharBuffer cb = buffer.asCharBuffer();
        char[] charArray = new char[cb.limit()];
        cb.get(charArray);
        return charArray;
    }
}
