package com.github.zengfr.easymodbus4j.common.util;

public class RegisterUtil {
    public static final String toHex(byte[] data) {
        return toHex(data, 0, data.length);
    }

    public static final String toHex(byte[] data, int off, int length) {
        StringBuffer buf = new StringBuffer(data.length * 2);
        for (int i = off; i < length; i++) {
            if ((data[i] & 0xFF) < 16)
                buf.append("0");
            buf.append(Long.toString((data[i] & 0xFF), 16));
            if (i < data.length - 1)
                buf.append(" ");
        }
        return buf.toString();
    }

    public static final byte[] toHex(int i) {
        StringBuffer buf = new StringBuffer(2);
        if ((i & 0xFF) < 16)
            buf.append("0");
        buf.append(Long.toString((i & 0xFF), 16).toUpperCase());
        return buf.toString().getBytes();
    }

    public static final int registerToUnsignedShort(byte[] bytes) {
        return (bytes[0] & 0xFF) << 8 | bytes[1] & 0xFF;
    }

    public static final byte[] unsignedShortToRegister(int v) {
        byte[] register = new byte[2];
        register[0] = (byte) (0xFF & v >> 8);
        register[1] = (byte) (0xFF & v);
        return register;
    }

    public static final short registerToShort(byte[] bytes) {
        return (short) (bytes[0] << 8 | bytes[1] & 0xFF);
    }

    public static final short registerToShort(byte[] bytes, int idx) {
        return (short) (bytes[idx] << 8 | bytes[idx + 1] & 0xFF);
    }

    public static final byte[] shortToRegister(short s) {
        byte[] register = new byte[2];
        register[0] = (byte) (0xFF & s >> 8);
        register[1] = (byte) (0xFF & s);
        return register;
    }

    public static final int registersToInt(byte[] bytes) {
        return (bytes[0] & 0xFF) << 24 | (bytes[1] & 0xFF) << 16 | (bytes[2] & 0xFF) << 8 | bytes[3] & 0xFF;
    }

    public static final byte[] intToRegisters(int v) {
        byte[] registers = new byte[4];
        registers[0] = (byte) (0xFF & v >> 24);
        registers[1] = (byte) (0xFF & v >> 16);
        registers[2] = (byte) (0xFF & v >> 8);
        registers[3] = (byte) (0xFF & v);
        return registers;
    }

    public static final long registersToLong(byte[] bytes) {
        return (long) (bytes[0] & 0xFF) << 56L | (long) (bytes[1] & 0xFF) << 48L | (long) (bytes[2] & 0xFF) << 40L | (long) (bytes[3] & 0xFF) << 32L | (long) (bytes[4] & 0xFF) << 24L | (bytes[5] & 0xFF) << 16L | (bytes[6] & 0xFF) << 8L | (bytes[7] & 0xFF);
    }

    public static final byte[] longToRegisters(long v) {
        byte[] registers = new byte[8];
        registers[0] = (byte) (int) (0xFFL & v >> 56L);
        registers[1] = (byte) (int) (0xFFL & v >> 48L);
        registers[2] = (byte) (int) (0xFFL & v >> 40L);
        registers[3] = (byte) (int) (0xFFL & v >> 32L);
        registers[4] = (byte) (int) (0xFFL & v >> 24L);
        registers[5] = (byte) (int) (0xFFL & v >> 16L);
        registers[6] = (byte) (int) (0xFFL & v >> 8L);
        registers[7] = (byte) (int) (0xFFL & v);
        return registers;
    }

    public static final float registersToFloat(byte[] bytes) {
        return Float.intBitsToFloat((bytes[0] & 0xFF) << 24 | (bytes[1] & 0xFF) << 16 | (bytes[2] & 0xFF) << 8 | bytes[3] & 0xFF);
    }

    public static final byte[] floatToRegisters(float f) {
        return intToRegisters(Float.floatToIntBits(f));
    }

    public static final double registersToDouble(byte[] bytes) {
        return Double.longBitsToDouble((long) (bytes[0] & 0xFF) << 56L | (long) (bytes[1] & 0xFF) << 48L | (long) (bytes[2] & 0xFF) << 40L | (long) (bytes[3] & 0xFF) << 32L | (long) (bytes[4] & 0xFF) << 24L | (bytes[5] & 0xFF) << 16L | (bytes[6] & 0xFF) << 8L | (bytes[7] & 0xFF));
    }

    public static final byte[] doubleToRegisters(double d) {
        return longToRegisters(Double.doubleToLongBits(d));
    }

    public static final int unsignedByteToInt(byte b) {
        return b & 0xFF;
    }

    public static final byte lowByte(int wd) {
        return (Integer.valueOf(0xFF & wd)).byteValue();
    }

    public static final byte hiByte(int wd) {
        return (Integer.valueOf(0xFF & wd >> 8)).byteValue();
    }

    public static final int makeWord(int hibyte, int lowbyte) {
        int hi = 0xFF & hibyte;
        int low = 0xFF & lowbyte;
        return hi << 8 | low;
    }
}
