package com.github.zengfr.easymodbus4j.func.response;

import com.github.zengfr.easymodbus4j.func.AbstractReadResponse;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

import java.util.HashMap;
import java.util.StringJoiner;

public class ErrorFunctionResponse extends AbstractReadResponse {
    private static final HashMap<Short, String> ERRORS = new HashMap<>();

    private short exceptionCode;

    private String exceptionMessage;

    static {
        ERRORS.put((short) 1, "ILLEGAL FUNCTION");
        ERRORS.put((short) 2, "ILLEGAL DATA ADDRESS");
        ERRORS.put((short) 3, "ILLEGAL DATA VALUE");
        ERRORS.put((short) 4, "SLAVE DEVICE FAILURE");
        ERRORS.put((short) 5, "ACKNOWLEDGE");
        ERRORS.put((short) 6, "SLAVE DEVICE BUSY");
        ERRORS.put((short) 8, "MEMORY PARITY ERROR");
        ERRORS.put((short) 10, "GATEWAY PATH UNAVAILABLE");
        ERRORS.put((short) 11, "GATEWAY TARGET DEVICE FAILED TO RESPOND");
    }

    public ErrorFunctionResponse(short functionCode) {
        super(functionCode);
    }

    public ErrorFunctionResponse(short functionCode, short exceptionCode) {
        super(functionCode);
        this.exceptionCode = exceptionCode;
        setExceptionMessage(this.exceptionCode);
    }

    private void setExceptionMessage(short exceptionCode) {
        this.exceptionMessage = (ERRORS.get(exceptionCode) != null) ? ERRORS.get(Short.valueOf(exceptionCode)) : "UNDEFINED ERROR";
    }

    public short getExceptionCode() {
        return this.exceptionCode;
    }

    public String getExceptionMessage() {
        return this.exceptionMessage;
    }

    public int calculateLength() {
        return 2;
    }

    public ByteBuf encode() {
        ByteBuf buf = Unpooled.buffer(calculateLength());
        buf.writeByte(getFunctionCode());
        buf.writeByte(this.exceptionCode);
        return buf;
    }

    public void decode(ByteBuf data) {
    }

    @Override
    public String toString() {
        return new StringJoiner(", ", ErrorFunctionResponse.class.getSimpleName() + "[", "]")
                .add("exceptionCode=" + exceptionCode)
                .add("functionCode=" + getFunctionCode())
                .add("exceptionMessage='" + exceptionMessage + "'")
                .toString();
    }
}
