package com.github.zengfr.easymodbus4j.func.response;

import com.github.zengfr.easymodbus4j.func.AbstractReadResponse;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

public class ReadInputRegistersResponse extends AbstractReadResponse {
    private short byteCount;

    private int[] inputRegisters;

    public ReadInputRegistersResponse() {
        super((short) 4);
    }

    public ReadInputRegistersResponse(int[] inputRegisters) {
        super((short) 4);
        if (inputRegisters.length > 125)
            throw new IllegalArgumentException();
        this.byteCount = (short) (inputRegisters.length * 2);
        this.inputRegisters = inputRegisters;
    }

    public int[] getInputRegisters() {
        return this.inputRegisters;
    }

    public short getByteCount() {
        return this.byteCount;
    }

    public int calculateLength() {
        return 2 + this.byteCount;
    }

    public ByteBuf encode() {
        ByteBuf buf = Unpooled.buffer(calculateLength());
        buf.writeByte(getFunctionCode());
        buf.writeByte(this.byteCount);
        for (int i = 0; i < this.inputRegisters.length; i++)
            buf.writeShort(this.inputRegisters[i]);
        return buf;
    }

    public void decode(ByteBuf data) {
        this.byteCount = data.readUnsignedByte();
        this.inputRegisters = new int[this.byteCount / 2];
        for (int i = 0; i < this.inputRegisters.length; i++)
            this.inputRegisters[i] = data.readUnsignedShort();
    }

    public String toString() {
        StringBuilder registers = new StringBuilder();
        registers.append("{");
        for (int i = 0; i < this.inputRegisters.length; i++) {
            registers.append(i);
            registers.append("=");
            registers.append(this.inputRegisters[i]);
            registers.append(",");
        }
        registers.delete(registers.length() - 1, registers.length());
        registers.append("}");
        return "ReadInputRegistersResponse{byteCount=" + this.byteCount + ", inputRegisters=" + registers + '}';
    }
}
