package com.github.zengfr.easymodbus4j.handle.impl;

import com.github.zengfr.easymodbus4j.func.request.*;
import com.github.zengfr.easymodbus4j.func.response.*;
import com.github.zengfr.easymodbus4j.handler.ModbusRequestHandler;
import com.github.zengfr.easymodbus4j.logging.ChannelLogger;
import com.github.zengfr.easymodbus4j.processor.ModbusSlaveRequestProcessor;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.github.zengfr.easymodbus4j.util.ModbusFrameUtil;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler.Sharable;

@Sharable
public class ModbusSlaveRequestHandler extends ModbusRequestHandler {
    private static final ChannelLogger logger = ChannelLogger.getLogger(ModbusSlaveRequestHandler.class);

    private final ModbusSlaveRequestProcessor processor;

    public short getTransactionIdentifierOffset() {
        return this.processor.getTransactionIdentifierOffset();
    }

    public ModbusSlaveRequestHandler(ModbusSlaveRequestProcessor processor) {
        this.processor = processor;
    }

    protected int getRespTransactionIdByReqTransactionId(int reqTransactionIdentifier) {
        return reqTransactionIdentifier + getTransactionIdentifierOffset();
    }

    protected ModbusFunction processRequestFrame(Channel channel, ModbusFrame frame) {
        if (this.processor.isShowFrameDetail())
            ModbusFrameUtil.showFrameLog(logger, channel, frame, true);
        return super.processRequestFrame(channel, frame);
    }

    protected WriteSingleCoilResponse writeSingleCoil(short unitIdentifier, WriteSingleCoilRequest request) {
        return this.processor.writeSingleCoil(unitIdentifier, request);
    }

    protected WriteSingleRegisterResponse writeSingleRegister(short unitIdentifier, WriteSingleRegisterRequest request) {
        return this.processor.writeSingleRegister(unitIdentifier, request);
    }

    protected ReadCoilsResponse readCoils(short unitIdentifier, ReadCoilsRequest request) {
        return this.processor.readCoils(unitIdentifier, request);
    }

    protected ReadDiscreteInputsResponse readDiscreteInputs(short unitIdentifier, ReadDiscreteInputsRequest request) {
        return this.processor.readDiscreteInputs(unitIdentifier, request);
    }

    protected ReadInputRegistersResponse readInputRegisters(short unitIdentifier, ReadInputRegistersRequest request) {
        return this.processor.readInputRegisters(unitIdentifier, request);
    }

    protected ReadHoldingRegistersResponse readHoldingRegisters(short unitIdentifier, ReadHoldingRegistersRequest request) {
        return this.processor.readHoldingRegisters(unitIdentifier, request);
    }

    protected WriteMultipleCoilsResponse writeMultipleCoils(short unitIdentifier, WriteMultipleCoilsRequest request) {
        return this.processor.writeMultipleCoils(unitIdentifier, request);
    }

    protected WriteMultipleRegistersResponse writeMultipleRegisters(short unitIdentifier, WriteMultipleRegistersRequest request) {
        return this.processor.writeMultipleRegisters(unitIdentifier, request);
    }
}
