package com.github.zengfr.easymodbus4j.handler;

import com.github.zengfr.easymodbus4j.channel.ChannelManager;
import com.github.zengfr.easymodbus4j.channel.ChannelReconnectable;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.EventLoop;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.logging.InternalLoggerFactory;

import java.util.concurrent.TimeUnit;

public abstract class ModbusInboundHandler extends SimpleChannelInboundHandler<ModbusFrame> {
    private static final InternalLogger logger = InternalLoggerFactory.getInstance(ModbusInboundHandler.class);

    private ChannelManager channelManager;

    static {
        logger.info("https://github.com/zengfr/easymodbus4j");
    }

    public void setChannelManager(ChannelManager channelManager) {
        this.channelManager = channelManager;
    }

    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        logger.error("error", cause);
        ctx.channel().close();
    }

    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        if (this.channelManager != null) {
            this.channelManager.removeChannel(ctx.channel());
            if (this.channelManager instanceof ChannelReconnectable) {
                final ChannelReconnectable rc = (ChannelReconnectable) this.channelManager;
                EventLoop eventLoop = ctx.channel().eventLoop();
                eventLoop.schedule(new Runnable() {
                    public void run() {
                        rc.reConnect();
                    }
                }, 5L, TimeUnit.SECONDS);
            }
        }
        super.channelInactive(ctx);
    }

    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        super.channelActive(ctx);
        if (this.channelManager != null)
            this.channelManager.addChannel(ctx.channel());
    }
}
