package com.github.zengfr.easymodbus4j.handler;

import com.github.zengfr.easymodbus4j.func.request.*;
import com.github.zengfr.easymodbus4j.func.response.*;
import com.github.zengfr.easymodbus4j.logging.ChannelLogger;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusHeader;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.util.concurrent.GenericFutureListener;

public abstract class ModbusRequestHandler extends ModbusInboundHandler {
    private static final ChannelLogger logger = ChannelLogger.getLogger(ModbusRequestHandler.class);

    protected void channelRead0(ChannelHandlerContext ctx, ModbusFrame frame) throws Exception {
        Channel channel = ctx.channel();
        logger.debug(channel, "channelRead0");
        ModbusFunction responseFunc = processRequestFrame(channel, frame);
        ModbusFrame responseFrame = buildResponseModbusFrame(frame, responseFunc);
        channel.write(responseFrame).addListener(ChannelFutureListener.CLOSE_ON_FAILURE);
    }

    protected ModbusFrame buildResponseModbusFrame(ModbusFrame reqFrame, ModbusFunction response) {
        int respTransactionId = getRespTransactionIdByReqTransactionId(reqFrame.getHeader().getTransactionIdentifier());
        ModbusHeader header = new ModbusHeader(respTransactionId, reqFrame.getHeader().getProtocolIdentifier(), response.calculateLength(), reqFrame.getHeader().getUnitIdentifier());
        return new ModbusFrame(header, response);
    }

    protected ModbusFunction processRequestFrame(Channel channel, ModbusFrame frame) {
        WriteMultipleCoilsResponse writeMultipleCoilsResponse = null;
        ModbusFunction function = frame.getFunction();
        short unitIdentifier = frame.getHeader().getUnitIdentifier();
        if (function instanceof WriteSingleCoilRequest) {
            WriteSingleCoilRequest request = (WriteSingleCoilRequest) function;
            WriteSingleCoilResponse writeSingleCoilResponse = writeSingleCoil(unitIdentifier, request);
        } else if (function instanceof WriteSingleRegisterRequest) {
            WriteSingleRegisterRequest request = (WriteSingleRegisterRequest) function;
            WriteSingleRegisterResponse writeSingleRegisterResponse = writeSingleRegister(unitIdentifier, request);
        } else if (function instanceof ReadCoilsRequest) {
            ReadCoilsRequest request = (ReadCoilsRequest) function;
            ReadCoilsResponse readCoilsResponse = readCoils(unitIdentifier, request);
        } else if (function instanceof ReadDiscreteInputsRequest) {
            ReadDiscreteInputsRequest request = (ReadDiscreteInputsRequest) function;
            ReadDiscreteInputsResponse readDiscreteInputsResponse = readDiscreteInputs(unitIdentifier, request);
        } else if (function instanceof ReadInputRegistersRequest) {
            ReadInputRegistersRequest request = (ReadInputRegistersRequest) function;
            ReadInputRegistersResponse readInputRegistersResponse = readInputRegisters(unitIdentifier, request);
        } else if (function instanceof ReadHoldingRegistersRequest) {
            ReadHoldingRegistersRequest request = (ReadHoldingRegistersRequest) function;
            ReadHoldingRegistersResponse readHoldingRegistersResponse = readHoldingRegisters(unitIdentifier, request);
        } else if (function instanceof WriteMultipleRegistersRequest) {
            WriteMultipleRegistersRequest request = (WriteMultipleRegistersRequest) function;
            WriteMultipleRegistersResponse writeMultipleRegistersResponse = writeMultipleRegisters(unitIdentifier, request);
        } else if (function instanceof WriteMultipleCoilsRequest) {
            WriteMultipleCoilsRequest request = (WriteMultipleCoilsRequest) function;
            writeMultipleCoilsResponse = writeMultipleCoils(unitIdentifier, request);
        } else {
            throw new UnsupportedOperationException("Function not supported!" + function);
        }
        return writeMultipleCoilsResponse;
    }

    protected abstract int getRespTransactionIdByReqTransactionId(int paramInt);

    protected abstract ReadCoilsResponse readCoils(short paramShort, ReadCoilsRequest paramReadCoilsRequest);

    protected abstract ReadDiscreteInputsResponse readDiscreteInputs(short paramShort, ReadDiscreteInputsRequest paramReadDiscreteInputsRequest);

    protected abstract ReadInputRegistersResponse readInputRegisters(short paramShort, ReadInputRegistersRequest paramReadInputRegistersRequest);

    protected abstract ReadHoldingRegistersResponse readHoldingRegisters(short paramShort, ReadHoldingRegistersRequest paramReadHoldingRegistersRequest);

    protected abstract WriteSingleCoilResponse writeSingleCoil(short paramShort, WriteSingleCoilRequest paramWriteSingleCoilRequest);

    protected abstract WriteSingleRegisterResponse writeSingleRegister(short paramShort, WriteSingleRegisterRequest paramWriteSingleRegisterRequest);

    protected abstract WriteMultipleCoilsResponse writeMultipleCoils(short paramShort, WriteMultipleCoilsRequest paramWriteMultipleCoilsRequest);

    protected abstract WriteMultipleRegistersResponse writeMultipleRegisters(short paramShort, WriteMultipleRegistersRequest paramWriteMultipleRegistersRequest);
}
