package com.github.zengfr.easymodbus4j.logging;

import io.netty.channel.Channel;
import io.netty.util.internal.logging.InternalLogger;
import io.netty.util.internal.logging.InternalLoggerFactory;
import org.slf4j.MDC;

public class ChannelLogger {
    protected final InternalLogger logger;

    public static ChannelLogger getLogger(Class<?> cls) {
        return new ChannelLogger(cls);
    }

    public ChannelLogger(Class<?> cls) {
        this.logger = InternalLoggerFactory.getInstance(cls);
    }

    protected String buildMsg(Channel channel, String format, Object... args) {
        if (args == null || args.length <= 0)
            return String.format("%s|%s", getChannelName(channel), format);
        return String.format("%s|%s", getChannelName(channel), String.format(format, args));
    }

    protected String buildMsg(Channel channel, Throwable ex) {
        return String.format("%s|%s", getChannelName(channel), ex.getMessage());
    }

    protected static String getChannelName(Channel channel) {
        if (channel != null && channel.remoteAddress() != null)
            return channel.remoteAddress().toString().replace('/', '.').replace(':', '.');
        return "null";
    }

    public static void putMDC(Channel channel) {
        MDC.put("channel", getChannelName(channel));
    }

    public void debug(Channel channel, String format, Object... args) {
        if (this.logger.isDebugEnabled()) {
            putMDC(channel);
            this.logger.debug(buildMsg(channel, format, args));
        }
    }

    public void info(Channel channel, String format, Object... args) {
        if (this.logger.isInfoEnabled()) {
            putMDC(channel);
            this.logger.info(buildMsg(channel, format, args));
        }
    }

    public void warn(Channel channel, String format, Object... args) {
        if (this.logger.isWarnEnabled()) {
            putMDC(channel);
            this.logger.warn(buildMsg(channel, format, args));
        }
    }

    public void error(Channel channel, Throwable ex) {
        if (this.logger.isErrorEnabled()) {
            putMDC(channel);
            this.logger.error(buildMsg(channel, ex), ex);
        }
    }
}
