package com.github.zengfr.easymodbus4j.processor;

import com.github.zengfr.easymodbus4j.func.AbstractRequest;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.util.ModbusFunctionUtil;

public abstract class AbstractModbusProcessor implements ModbusProcessor {
    private final short transactionIdentifierOffset;

    private final boolean isShowFrameDetail;

    public AbstractModbusProcessor() {
        this((short) 0, true);
    }

    public AbstractModbusProcessor(short transactionIdentifierOffset, boolean isShowFrameDetail) {
        this.transactionIdentifierOffset = transactionIdentifierOffset;
        this.isShowFrameDetail = isShowFrameDetail;
    }

    protected boolean isRequestResponseMatch(AbstractRequest reqFunc, ModbusFunction respFunc) {
        return (reqFunc != null && respFunc != null && reqFunc.getFunctionCode() == respFunc.getFunctionCode());
    }

    protected boolean isRequestResponseValueMatch(AbstractRequest reqFunc, ModbusFunction respFunc) {
        byte[] respFuncValuesArray = ModbusFunctionUtil.getFunctionValues(respFunc);
        return isRequestResponseValueMatch(reqFunc, respFuncValuesArray);
    }

    protected boolean isRequestResponseValueMatch(AbstractRequest reqFunc, byte[] respFuncValuesArray) {
        if (reqFunc == null)
            return false;
        int quantityOfInputRegisters = reqFunc.getValue();
        return (quantityOfInputRegisters * 2 == respFuncValuesArray.length || (respFuncValuesArray.length == 1 && quantityOfInputRegisters == respFuncValuesArray.length));
    }

    public short getTransactionIdentifierOffset() {
        return this.transactionIdentifierOffset;
    }

    public boolean isShowFrameDetail() {
        return this.isShowFrameDetail;
    }
}
