package com.github.zengfr.easymodbus4j.protocol.rtu;

import com.github.zengfr.easymodbus4j.common.util.RtuCrcUtil;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

public class ModbusRtuFrame {
    private final short unitIdentifier;

    private final ModbusFunction function;

    public ModbusRtuFrame(short unitIdentifier, ModbusFunction function) {
        this.unitIdentifier = unitIdentifier;
        this.function = function;
    }

    public short getUnitId() {
        return this.unitIdentifier;
    }

    public ModbusFunction getFunction() {
        return this.function;
    }

    public ByteBuf encode() {
        ByteBuf buf = Unpooled.buffer();
        buf.writeByte(this.unitIdentifier);
        buf.writeBytes(this.function.encode());
        writeRtuCRC(buf);
        return buf;
    }

    public static void writeRtuCRC(ByteBuf buffer) {
        int startReaderIndex = buffer.readerIndex();
        int crc = RtuCrcUtil.calculateCRC(buffer);
        buffer.readerIndex(startReaderIndex);
        buffer.writeByte((byte) (0xFF & crc >> 8));
        buffer.writeByte((byte) (0xFF & crc));
    }

    public String toString() {
        return this.unitIdentifier + " " + this.function;
    }
}
