package com.github.zengfr.easymodbus4j.protocol.tcp;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

public class ModbusHeader {
    private final int transactionIdentifier;

    private final int protocolIdentifier;

    private final int length;

    private final short unitIdentifier;

    public ModbusHeader(int transactionIdentifier, int protocolIdentifier, int pduLength, short unitIdentifier) {
        this.transactionIdentifier = transactionIdentifier;
        this.protocolIdentifier = protocolIdentifier;
        this.length = pduLength + 1;
        this.unitIdentifier = unitIdentifier;
    }

    public int getLength() {
        return this.length;
    }

    public int getProtocolIdentifier() {
        return this.protocolIdentifier;
    }

    public int getTransactionIdentifier() {
        return this.transactionIdentifier;
    }

    public short getUnitIdentifier() {
        return this.unitIdentifier;
    }

    public static ModbusHeader decode(ByteBuf buffer) {
        return new ModbusHeader(buffer.readUnsignedShort(), buffer
                .readUnsignedShort(), buffer
                .readUnsignedShort() - 1, buffer
                .readUnsignedByte());
    }

    public ByteBuf encode() {
        ByteBuf buf = Unpooled.buffer();
        buf.writeShort(this.transactionIdentifier);
        buf.writeShort(this.protocolIdentifier);
        buf.writeShort(this.length);
        buf.writeByte(this.unitIdentifier);
        return buf;
    }

    public String toString() {
        return "T=" + this.transactionIdentifier + " P=" + this.protocolIdentifier + " L=" + this.length + " U=" + this.unitIdentifier;
    }
}
