package com.github.zengfr.easymodbus4j.queue;

import com.github.zengfr.easymodbus4j.common.util.ScheduledUtil;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.DelayQueue;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class AutoClearConcurrentLinkedQueue<T> extends ConcurrentLinkedQueue<T> {
    private static final long serialVersionUID = -4124978116278119617L;

    protected DelayQueue<DelayedItem<T>> delayQueue;

    private final long timeout;

    private final ReadWriteLock readWriteLock;

    private final Lock readLock;

    public AutoClearConcurrentLinkedQueue(long timeout) {
        this.timeout = timeout;
        this.delayQueue = new DelayQueue<>();
        this.readWriteLock = new ReentrantReadWriteLock(true);
        this.readLock = this.readWriteLock.readLock();
        Runnable runnable = () -> execute4AutoClear();
        ScheduledUtil.schedule(runnable, 10L);
    }

    private void execute4AutoClear() {
        while (true) {
            try {
                this.readLock.lock();
                DelayedItem<T> delayItem = this.delayQueue.take();
                T item = peek();
                if (delayItem.getItem().equals(item))
                    super.poll();
                this.readLock.unlock();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    protected DelayedItem<T> getDelayedItem(T item) {
        return new DelayedItem<>(item, this.timeout * (size() + 1));
    }

    public boolean add(T e) {
        this.delayQueue.add(getDelayedItem(e));
        return super.add(e);
    }

    public boolean offer(T e) {
        this.delayQueue.offer(getDelayedItem(e));
        return super.offer(e);
    }

    public T poll() {
        this.readLock.lock();
        this.delayQueue.poll();
        T item = super.poll();
        this.readLock.unlock();
        return item;
    }
}
