package com.github.zengfr.easymodbus4j.registers;

import com.github.zengfr.easymodbus4j.common.RegisterOrder;
import com.github.zengfr.easymodbus4j.common.util.IntArrayUtil;
import com.github.zengfr.easymodbus4j.common.util.RegistersUtil;
import com.google.common.collect.Lists;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class Registers {
    private final Map<Integer, Boolean> coils = new HashMap<>();

    private final Map<Integer, Boolean> inputs = new HashMap<>();

    private final Map<Integer, Short> holdingRegisters = new HashMap<>();

    private final Map<Integer, Short> inputRegisters = new HashMap<>();

    public void setHoldingRegister(int offset, float[] values) {
        if (validateOffset(offset)) {
            ArrayList<Integer> list = Lists.newArrayList();
            for (int i = 0; i < values.length; i++) {
                int[] vv = RegistersUtil.convertFloatToRegisters(values[i], RegisterOrder.HighLow);
                int[] arrayOfInt1;
                int j;
                byte b;
                for (arrayOfInt1 = vv, j = arrayOfInt1.length, b = 0; b < j; ) {
                    Integer v = Integer.valueOf(arrayOfInt1[b]);
                    list.add(v);
                    b++;
                }
            }
            int[] registers = IntArrayUtil.toIntArray(list);
            setHoldingRegister(offset, registers);
        }
    }

    public void setHoldingRegister(int offset, Integer... values) {
        if (validateOffset(offset)) {
            ArrayList<Integer> list = Lists.newArrayList();
            for (int i = 0; i < values.length; i++) {
                int[] vv = RegistersUtil.convertIntToRegisters(values[i].intValue(), RegisterOrder.HighLow);
                int[] arrayOfInt1;
                int j;
                byte b;
                for (arrayOfInt1 = vv, j = arrayOfInt1.length, b = 0; b < j; ) {
                    Integer v = Integer.valueOf(arrayOfInt1[b]);
                    list.add(v);
                    b++;
                }
            }
            int[] registers = IntArrayUtil.toIntArray(list);
            setHoldingRegister(offset, registers);
        }
    }

    protected void setHoldingRegister(int offset, int[] registers) {
        if (validateOffset(offset))
            for (int i = 0; i < registers.length; i++)
                setHoldingRegister(offset + i, (short) registers[i]);
    }

    public void setHoldingRegister(int offset, short[] registers) {
        if (validateOffset(offset))
            for (int i = 0; i < registers.length; i++)
                setHoldingRegister(offset + i, registers[i]);
    }

    public void setInputRegister(int offset, short[] registers) {
        if (validateOffset(offset))
            for (int i = 0; i < registers.length; i++)
                setInputRegister(offset + i, registers[i]);
    }

    protected boolean validateOffset(int offset) {
        return offset >= 0 && offset <= 65535;
    }

    public synchronized void setInputRegister(int offset, short value) {
        if (validateOffset(offset))
            this.inputRegisters.put(Integer.valueOf(offset), Short.valueOf(value));
    }

    public synchronized void setHoldingRegister(int offset, short value) {
        if (validateOffset(offset))
            this.holdingRegisters.put(Integer.valueOf(offset), Short.valueOf(value));
    }

    public synchronized void setInput(int offset, boolean value) {
        if (validateOffset(offset))
            this.inputs.put(Integer.valueOf(offset), Boolean.valueOf(value));
    }

    public synchronized void setCoil(int offset, boolean value) {
        if (validateOffset(offset))
            this.coils.put(Integer.valueOf(offset), Boolean.valueOf(value));
    }

    public boolean getCoil(int offset) {
        return getBoolean(offset, this.coils);
    }

    public boolean getInput(int offset) {
        return getBoolean(offset, this.inputs);
    }

    public short getInputRegister(int offset) {
        return getShort(offset, this.inputRegisters);
    }

    public float getInputRegister4Float(int offset) {
        return getRegister4Float(offset, this.inputRegisters);
    }

    public int getInputRegister4Int(int offset) {
        return getRegister4Int(offset, this.inputRegisters);
    }

    public short getHoldingRegister(int offset) {
        return getShort(offset, this.holdingRegisters);
    }

    public float getHoldingRegister4Float(int offset) {
        return getRegister4Float(offset, this.holdingRegisters);
    }

    public int getHoldingRegister4Int(int offset) {
        return getRegister4Int(offset, this.holdingRegisters);
    }

    public int[] getInputRegisters(int offset, int registersCount) {
        return getRegisters(offset, this.inputRegisters, registersCount);
    }

    public int[] getHoldingRegisters(int offset, int registersCount) {
        return getRegisters(offset, this.holdingRegisters, registersCount);
    }

    protected float getRegister4Float(int offset, Map<Integer, Short> map) {
        int[] registers = getRegisters(offset, map, 4);
        return RegistersUtil.convertRegistersToFloat(registers);
    }

    protected int getRegister4Int(int offset, Map<Integer, Short> map) {
        int[] registers = getRegisters(offset, map, 2);
        return RegistersUtil.convertRegistersToInt(registers);
    }

    protected int[] getRegisters(int offset, Map<Integer, Short> map, int registersCount) {
        int[] rs = new int[registersCount];
        for (int i = 0; i < registersCount; i++)
            rs[i] = getShort(offset + i, map);
        return rs;
    }

    protected short getShort(int offset, Map<Integer, Short> map) {
        Short value = map.get(Integer.valueOf(offset));
        if (value == null)
            return 0;
        return value.shortValue();
    }

    protected boolean getBoolean(int offset, Map<Integer, Boolean> map) {
        Boolean value = map.get(Integer.valueOf(offset));
        if (value == null)
            return false;
        return value.booleanValue();
    }
}
