package com.github.zengfr.easymodbus4j.schedule;

import com.github.zengfr.easymodbus4j.common.util.ScheduledUtil;
import com.github.zengfr.easymodbus4j.sender.util.ModbusRequestSendUtil;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import io.netty.channel.Channel;
import io.netty.util.internal.logging.InternalLogger;

import java.util.Collection;
import java.util.List;
import java.util.concurrent.TimeUnit;

public abstract class ModbusMasterSchedule {
    private final Cache<Integer, List<String>> cache = CacheBuilder.newBuilder().expireAfterWrite(300000L, TimeUnit.MILLISECONDS).build();

    protected abstract ModbusRequestSendUtil.PriorityStrategy getPriorityStrategy();

    protected abstract List<String> buildReqsList();

    protected abstract int getFixedDelay();

    protected abstract InternalLogger getLogger();

    protected List<String> getReqsListByCache() {
        Integer key = Integer.valueOf(0);
        List<String> reqStrs = this.cache.getIfPresent(key);
        if (reqStrs == null || reqStrs.isEmpty()) {
            reqStrs = buildReqsList();
            this.cache.put(key, reqStrs);
        }
        return reqStrs;
    }

    public void schedule(Collection<Channel> channels, int sleep) {
        getLogger().debug(String.format("schedule->channels:%s,sleep:%s ms", Integer.valueOf(channels.size()), Integer.valueOf(sleep)));
        sendRequests4Auto(channels, sleep);
    }

    public void run(Collection<Channel> channels) {
        List<String> reqStrs = buildReqsList();
        getLogger().debug(String.format("channels:%s,reqStrs:%s", Integer.valueOf(channels.size()), Integer.valueOf(reqStrs.size())));
        ModbusRequestSendUtil.sendRequests(channels, reqStrs, true, getFixedDelay(), getPriorityStrategy());
    }

    private void sendRequests4Auto(Collection<Channel> channels, int sleep) {
        Runnable r = () -> {
            try {
                List<String> reqStrs = getReqsListByCache();
                getLogger().debug(String.format("channels:%s,reqStrs:%s,sleep:%s ms", Integer.valueOf(channels.size()), Integer.valueOf(reqStrs.size()), Integer.valueOf(sleep)));
                ModbusRequestSendUtil.sendRequests(channels, reqStrs, true, getFixedDelay(), getPriorityStrategy());
            } catch (Exception ex) {
                getLogger().error(ex);
            }
        };
        ScheduledUtil.scheduleWithFixedDelay(r, sleep);
    }
}
