package com.github.zengfr.easymodbus4j.sender;

import com.github.zengfr.easymodbus4j.ModbusConsts;
import com.github.zengfr.easymodbus4j.func.request.*;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import io.netty.channel.Channel;

import java.util.BitSet;

public class ChannelAsyncSender extends AbstractChannelSender {
    public ChannelAsyncSender(Channel channel) {
        this(channel, ModbusConsts.DEFAULT_UNIT_IDENTIFIER, (short) 0);
    }

    public ChannelAsyncSender(Channel channel, short unitIdentifier) {
        this(channel, unitIdentifier, (short) 0);
    }

    public ChannelAsyncSender(Channel channel, short unitId, short protocolIdentifier) {
        super(channel, unitId, protocolIdentifier);
    }

    public int writeSingleCoilAsync(int address, boolean state) throws Exception {
        return callModbusFunction(new WriteSingleCoilRequest(address, state));
    }

    public int writeSingleRegisterAsync(int address, int value) throws Exception {
        return callModbusFunction(new WriteSingleRegisterRequest(address, value));
    }

    public int readCoilsAsync(int startAddress, int quantityOfCoils) throws Exception {
        return callModbusFunction(new ReadCoilsRequest(startAddress, quantityOfCoils));
    }

    public int readDiscreteInputsAsync(int startAddress, int quantityOfCoils) throws Exception {
        return callModbusFunction(new ReadDiscreteInputsRequest(startAddress, quantityOfCoils));
    }

    public int readInputRegistersAsync(int startAddress, int quantityOfInputRegisters) throws Exception {
        return callModbusFunction(new ReadInputRegistersRequest(startAddress, quantityOfInputRegisters));
    }

    public int readHoldingRegistersAsync(int startAddress, int quantityOfInputRegisters) throws Exception {
        return callModbusFunction(new ReadHoldingRegistersRequest(startAddress, quantityOfInputRegisters));
    }

    public int writeMultipleCoilsAsync(int address, int quantityOfOutputs, boolean[] outputsValue) throws Exception {
        return callModbusFunction(new WriteMultipleCoilsRequest(address, quantityOfOutputs, outputsValue));
    }

    public int writeMultipleCoilsAsync(int address, int quantityOfOutputs, BitSet outputsValue) throws Exception {
        return callModbusFunction(new WriteMultipleCoilsRequest(address, quantityOfOutputs, outputsValue));
    }

    public int writeMultipleRegistersAsync(int address, int quantityOfRegisters, int[] registers) throws Exception {
        return callModbusFunction(new WriteMultipleRegistersRequest(address, quantityOfRegisters, registers));
    }
}
