package com.github.zengfr.easymodbus4j.sender;

import com.github.zengfr.easymodbus4j.ModbusConsts;
import com.github.zengfr.easymodbus4j.func.request.*;
import com.github.zengfr.easymodbus4j.func.response.*;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import io.netty.channel.Channel;

import java.util.BitSet;

public class ChannelSyncSender extends AbstractChannelSender {
    public ChannelSyncSender(Channel channel) {
        this(channel, ModbusConsts.DEFAULT_UNIT_IDENTIFIER, (short) 0);
    }

    public ChannelSyncSender(Channel channel, short unitIdentifier) {
        this(channel, unitIdentifier, (short) 0);
    }

    public ChannelSyncSender(Channel channel, short unitId, short protocolIdentifier) {
        super(channel, unitId, protocolIdentifier);
    }

    public WriteSingleCoilResponse writeSingleCoil(int address, boolean state) throws Exception {
        return callModbusFunctionSync(new WriteSingleCoilRequest(address, state));
    }

    public WriteSingleRegisterResponse writeSingleRegister(int address, int value) throws Exception {
        return callModbusFunctionSync(new WriteSingleRegisterRequest(address, value));
    }

    public ReadCoilsResponse readCoils(int startAddress, int quantityOfCoils) throws Exception {
        return callModbusFunctionSync(new ReadCoilsRequest(startAddress, quantityOfCoils));
    }

    public ReadDiscreteInputsResponse readDiscreteInputs(int startAddress, int quantityOfCoils) throws Exception {
        return callModbusFunctionSync(new ReadDiscreteInputsRequest(startAddress, quantityOfCoils));
    }

    public ReadInputRegistersResponse readInputRegisters(int startAddress, int quantityOfInputRegisters) throws Exception {
        return callModbusFunctionSync(new ReadInputRegistersRequest(startAddress, quantityOfInputRegisters));
    }

    public ReadHoldingRegistersResponse readHoldingRegisters(int startAddress, int quantityOfInputRegisters) throws Exception {
        return callModbusFunctionSync(new ReadHoldingRegistersRequest(startAddress, quantityOfInputRegisters));
    }

    public WriteMultipleCoilsResponse writeMultipleCoils(int address, int quantityOfOutputs, boolean[] outputsValue) throws Exception {
        return callModbusFunctionSync(new WriteMultipleCoilsRequest(address, quantityOfOutputs, outputsValue));
    }

    public WriteMultipleCoilsResponse writeMultipleCoils(int address, int quantityOfOutputs, BitSet outputsValue) throws Exception {
        return callModbusFunctionSync(new WriteMultipleCoilsRequest(address, quantityOfOutputs, outputsValue));
    }

    public WriteMultipleRegistersResponse writeMultipleRegisters(int address, int quantityOfRegisters, int[] registers) throws Exception {
        return callModbusFunctionSync(new WriteMultipleRegistersRequest(address, quantityOfRegisters, registers));
    }
}
