package com.github.zengfr.easymodbus4j.sender;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

public class SyncFuture<T> implements Future<T> {
    private final CountDownLatch latch = new CountDownLatch(1);

    private T response;

    private final long beginTime = System.currentTimeMillis();

    public boolean cancel(boolean mayInterruptIfRunning) {
        return false;
    }

    public boolean isCancelled() {
        return false;
    }

    public boolean isDone() {
        return this.response != null;
    }

    public T get() throws InterruptedException {
        this.latch.await();
        return this.response;
    }

    public T get(long timeout, TimeUnit unit) throws InterruptedException {
        if (this.latch.await(timeout, unit))
            return this.response;
        return null;
    }

    public void setResponse(T response) {
        this.response = response;
        this.latch.countDown();
    }

    public long getBeginTime() {
        return this.beginTime;
    }
}
