package com.github.zengfr.easymodbus4j.util;

import com.github.zengfr.easymodbus4j.common.util.ByteBufUtil;
import com.github.zengfr.easymodbus4j.common.util.ByteUtil;
import com.github.zengfr.easymodbus4j.common.util.HexUtil;
import com.github.zengfr.easymodbus4j.common.util.PadUtil;
import com.github.zengfr.easymodbus4j.logging.ChannelLogger;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import io.netty.channel.Channel;
import io.netty.util.internal.logging.InternalLogger;
import org.apache.commons.lang3.StringUtils;

import java.util.BitSet;

public class ModbusFrameUtil {
    private static final String splitLine = PadUtil.padLeft("-", 128, '-');

    public static void showFrameLog(ChannelLogger logger, Channel channel, ModbusFrame frame, boolean isShowHex) {
        logger.debug(channel, getFrameLog(channel, frame, isShowHex));
    }

    public static void showFrameLog(InternalLogger logger, Channel channel, ModbusFrame frame, boolean isShowHex) {
        logger.debug(getFrameLog(channel, frame, isShowHex));
    }

    public static String getFrameLog(Channel channel, ModbusFrame frame, boolean isShowHex) {
        byte[] bytes = ByteBufUtil.toBytes(frame.encode());
        String v1 = HexUtil.bytesToHexString(bytes, " ");
        String v2 = frame.toString();
        boolean isResponseFrame = frame.getFunction().toString().contains("Response");
        int transactionId = frame.getHeader().getTransactionIdentifier();
        StringBuilder sBuilder = new StringBuilder();
        sBuilder.append(String.format("%s", getChannelAddress(channel)));
        if (isShowHex) {
            sBuilder.append(String.format("\r\n%s", splitLine));
            sBuilder.append(String.format("\r\n%s", v1));
        }
        sBuilder.append(String.format("\r\n%s", splitLine));
        if (isResponseFrame) {
            sBuilder.append(String.format("\r\n%s %s", (transactionId < 0) ? "RTU" : "TCP", v2));
            sBuilder.append(String.format("\r\n%s", splitLine));
            sBuilder.append(getFunctionLog(frame.getFunction()));
        } else {
            sBuilder.append(String.format("\r\n%s", v2));
            sBuilder.append(String.format("\r\n%s", splitLine));
        }
        return sBuilder.toString();
    }

    public static String getFunctionLog(ModbusFunction function) {
        StringBuilder sBuilder = new StringBuilder();
        byte[] valuesArray = ModbusFunctionUtil.getFunctionValues(function);
        sBuilder.append(String.format("\r\nhex    :%s", HexUtil.bytesToHexString(valuesArray, " ")));
        sBuilder.append(String.format("\r\nbitset :%s", BitSet.valueOf(valuesArray)));
        sBuilder.append(String.format("\r\nbyte   :%s", StringUtils.join(valuesArray, ',')));
        sBuilder.append(String.format("\r\nshort  :%s", StringUtils.join(ByteUtil.toShortArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nushort :%s", StringUtils.join(ByteUtil.toUShortArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nint    :%s", StringUtils.join(ByteUtil.toIntArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nlong   b:%s", StringUtils.join(ByteUtil.toLongArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nfloat  b:%s", StringUtils.join(ByteUtil.toFloatArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\ndouble b:%s", StringUtils.join(ByteUtil.toDoubleArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nchar   b:%s", StringUtils.join(ByteUtil.toCharArray(valuesArray), ',')));
        sBuilder.append(String.format("\r\nlong   l:%s", StringUtils.join(ByteUtil.toLongArray(valuesArray, false), ',')));
        sBuilder.append(String.format("\r\nfloat  l:%s", StringUtils.join(ByteUtil.toFloatArray(valuesArray, false), ',')));
        sBuilder.append(String.format("\r\ndouble l:%s", StringUtils.join(ByteUtil.toDoubleArray(valuesArray, false), ',')));
        sBuilder.append(String.format("\r\nchar   l:%s", StringUtils.join(ByteUtil.toCharArray(valuesArray, false), ',')));
        sBuilder.append(String.format("\r\n%s", splitLine));
        return sBuilder.toString();
    }

    private static String getChannelAddress(Channel channel) {
        return channel.remoteAddress().toString();
    }
}
