package tofly.modbus_gateway2;

import com.github.zengfr.easymodbus4j.channel.ModbusRtuChannelInitializer;
import com.github.zengfr.easymodbus4j.codec.ModbusPduRespCodec;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.github.zengfr.easymodbus4j.server.ModbusServer4TcpMaster;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.logging.LoggingHandler;
import org.springframework.context.ApplicationEventPublisher;
import tofly.modbus_gateway2.config.ModbusServerProperties;

@Sharable
public class GwModbusTcpChannelInitializer extends ModbusRtuChannelInitializer {

    private final ModbusServer4TcpMaster modbusServer4TcpMaster;
    private final ApplicationEventPublisher eventPublisher;

    private final ModbusServerProperties modbusServerProperties;
    public GwModbusTcpChannelInitializer(Boolean isSlave,
                                         SimpleChannelInboundHandler<ModbusFrame> handler,
                                         GwModbusServer4TcpMaster modbusServer4Master,
                                         ApplicationEventPublisher eventPublisher,
                                         ModbusServerProperties modbusServerProperties) {
        super(isSlave, handler);
        this.modbusServer4TcpMaster = modbusServer4Master;
        this.modbusServerProperties = modbusServerProperties;
        try {
            this.modbusServer4TcpMaster.setup(this);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        this.eventPublisher = eventPublisher;
    }

    @Override
    protected void initPipelinePreProcessHandler(ChannelPipeline pipeline) {
        pipeline.addLast("logging-framer", new LoggingHandler());
        pipeline.addLast("first-framer", new FirstFrameDecoder(modbusServer4TcpMaster, eventPublisher, modbusServerProperties));
        pipeline.addLast("heartbeat-framer", new HeartbeatFrameDecoder(modbusServer4TcpMaster, eventPublisher));
        pipeline.addLast("codec", new ModbusRtuCodec2(new ModbusPduRespCodec()));
    }
}
