package tofly.modbus_gateway2;

import com.github.zengfr.easymodbus4j.server.ModbusServer4TcpMaster;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import io.netty.util.AttributeKey;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEventPublisher;

import java.nio.charset.Charset;
import java.util.List;

@Slf4j
public class HeartbeatFrameDecoder extends ByteToMessageDecoder {


    public final static AttributeKey<Long> ATTR_KEY_ALIVE_LAST_TIME = AttributeKey.valueOf("heartbeat_last_time");

    private final ModbusServer4TcpMaster modbusServer4TcpMaster;
    private final ApplicationEventPublisher eventPublisher;


    public HeartbeatFrameDecoder(ModbusServer4TcpMaster modbusServer4TcpMaster, ApplicationEventPublisher eventPublisher) {
        this.modbusServer4TcpMaster = modbusServer4TcpMaster;
        this.eventPublisher = eventPublisher;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf byteBuf, List<Object> list) throws Exception {
        int readableBytes = byteBuf.readableBytes();
        ctx.channel().attr(ATTR_KEY_ALIVE_LAST_TIME).set(System.currentTimeMillis());
        if (readableBytes == 1) {
            CharSequence charSequence = byteBuf.readCharSequence(readableBytes, Charset.forName("US-ASCII"));
            String value = charSequence.toString();
            if("$".equals(value)) {
                log.trace("心跳包:{}", value);
            }
        } else {
            ByteBuf frame = byteBuf.readBytes(readableBytes);
            list.add(frame);
        }
    }
}
