
package tofly.modbus_gateway2;

import com.github.zengfr.easymodbus4j.codec.ModbusPduCodec;
import com.github.zengfr.easymodbus4j.codec.rtu.ModbusRtuCodecUtil;
import com.github.zengfr.easymodbus4j.common.util.RtuCrcUtil;
import com.github.zengfr.easymodbus4j.func.response.ErrorFunctionResponse;
import com.github.zengfr.easymodbus4j.protocol.ModbusFunction;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusFrame;
import com.github.zengfr.easymodbus4j.protocol.tcp.ModbusHeader;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageCodec;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
public class ModbusRtuCodec2 extends ByteToMessageCodec<ModbusFrame> {
    protected final ModbusPduCodec pduCodec;

    public ModbusRtuCodec2(ModbusPduCodec pduCodec) {
        this.pduCodec = pduCodec;
    }

    protected void encode(ChannelHandlerContext ctx, ModbusFrame msg, ByteBuf out) throws Exception {
        ByteBuf sendBuf = ctx.alloc().heapBuffer(1 + (msg.getHeader().getLength() - 1) + 2);
        sendBuf.writeByte(msg.getHeader().getUnitIdentifier());
        sendBuf.writeBytes(msg.getFunction().encode());
        this.writeRtuCRC(sendBuf);
        ctx.writeAndFlush(sendBuf);
    }

    private void writeRtuCRC(ByteBuf buffer) {
        int startReaderIndex = buffer.readerIndex();
        int crc = RtuCrcUtil.calculateCRC(buffer);
        buffer.readerIndex(startReaderIndex);
        buffer.writeByte((byte)(255 & crc >> 8));
        buffer.writeByte((byte)(255 & crc));
    }

    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        for(; in.readableBytes() >= ModbusRtuCodecUtil.getMessageMinSize();) {
            short unitId = in.readUnsignedByte();
            ModbusFunction function = this.pduCodec.decode(in);
            if(function instanceof ErrorFunctionResponse){
                log.info("无效数据:{}", function);
                in.clear();
                return;
            }
            int crc = in.readUnsignedShort();
            int transactionId = -crc;
            int pduLength = function.calculateLength();
            int protocolIdentifier = 0;
            ModbusHeader mbapHeader = new ModbusHeader(transactionId, protocolIdentifier, pduLength, unitId);
            ModbusFrame frame = new ModbusFrame(mbapHeader, function);
            out.add(frame);
        }

    }
}
