package tofly.modbus_gateway2.config;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalUnit;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Created by huangdecai on 2023/9/1.
 * MODBUS server configuration
 */
@Configuration
@ConfigurationProperties(prefix = "modbus.gateway2.server")
public class ModbusServerProperties {

    private int port;
    private String table;

    private String monitorTable = "device_monitor";
    private String monitorHistoryTable = "device_monitor_history";

    private boolean store = true;

    //保活超时时间
    private Duration aliveTimeout = Duration.of(2, ChronoUnit.MINUTES);

    private Map<String, SlaveConfiguration> slaveMappings = new LinkedHashMap<>();

    public ModbusServerProperties() {
        super();
        this.port = 502;
        this.table = "tsdata";
    }

    public Map<String, SlaveConfiguration> getSlaveMappings() {
        return slaveMappings;
    }

    public void setSlaveMappings(Map<String, SlaveConfiguration> slaveMappings) {
        this.slaveMappings = slaveMappings;
    }

    public int getPort() {
        return port;
    }

    public void setPort(int port) {
        this.port = port;
    }

    public String getTable() {
        return table;
    }

    public void setTable(String table) {
        this.table = table;
    }

    public String getMonitorTable() {
        return monitorTable;
    }

    public void setMonitorTable(String monitorTable) {
        this.monitorTable = monitorTable;
    }

    public String getMonitorHistoryTable() {
        return monitorHistoryTable;
    }

    public void setMonitorHistoryTable(String monitorHistoryTable) {
        this.monitorHistoryTable = monitorHistoryTable;
    }

    public boolean isStore() {
        return store;
    }

    public void setStore(boolean store) {
        this.store = store;
    }

    public Duration getAliveTimeout() {
        return aliveTimeout;
    }

    public void setAliveTimeout(Duration aliveTimeout) {
        this.aliveTimeout = aliveTimeout;
    }

    public static class SlaveConfiguration {
        private final static int DEFAULT_PERIOD = 1000;

        private int period;

        private Map<String, DeviceConfiguration> deviceMappings = new LinkedHashMap<>();

        public SlaveConfiguration() {
            this.period = DEFAULT_PERIOD;
        }

        public int getPeriod() {
            return period;
        }

        public void setPeriod(int period) {
            this.period = period;
        }

        public Map<String, DeviceConfiguration> getDeviceMappings() {
            return deviceMappings;
        }

        public void setDeviceMappings(Map<String, DeviceConfiguration> deviceMappings) {
            this.deviceMappings = deviceMappings;
        }
    }

    public static class DeviceConfiguration {
        private Short unitIdentifier;

        private Integer functionCode;

        private Map<String, FieldConfiguration> fieldMappings = new LinkedHashMap<>();

        public DeviceConfiguration() {
            this.unitIdentifier = 1;
            this.functionCode = 3;
        }

        public Short getUnitIdentifier() {
            return unitIdentifier;
        }

        public void setUnitIdentifier(Short unitIdentifier) {
            this.unitIdentifier = unitIdentifier;
        }

        public Map<String, FieldConfiguration> getFieldMappings() {
            return fieldMappings;
        }

        public void setFieldMappings(Map<String, FieldConfiguration> fieldMappings) {
            this.fieldMappings = fieldMappings;
        }

        public Integer getFunctionCode() {
            return functionCode;
        }

        public void setFunctionCode(Integer functionCode) {
            this.functionCode = functionCode;
        }
    }

    @Data
    public static class FieldConfiguration {
        private String name;
        // type=0是以下参数都无效
        private int type;
        // 保留小时位数
        private int digits = 2;
        // type=-1时，以下参数生效
        private int upper = 2;
        private int lower = 0;
        private int gcUpper = 5000;
        private int gcLower = 50;
    }


}
