package tofly.modbus_gateway2.log;

import java.lang.reflect.Constructor;

public class LogFactory {
    private static Constructor logConstructor;

    static {
        tryImplementation("org.slf4j.Logger", "tofly.modbus_gateway2.log.SLF4JImpl");
    }

    private static void tryImplementation(String p_strTestClassName, String p_strImplClassName) {
        if (logConstructor != null) {
            return;
        }
        try {
            getClassByName(p_strTestClassName);
            Class implClass = getClassByName(p_strImplClassName);
            logConstructor = implClass.getConstructor(new Class[]{String.class});

            Class<?> declareClass = logConstructor.getDeclaringClass();
            if (!Log.class.isAssignableFrom(declareClass)) {
                logConstructor = null;
            }

            try {
                if (null != logConstructor) {
                    logConstructor.newInstance(LogFactory.class.getName());
                }
            } catch (Throwable t) {
                logConstructor = null;
            }

        } catch (Throwable t) {
            // skip
        }
    }

    public static Log getLog(Class p_clzIn) {
        return getLog(p_clzIn.getName());
    }

    private static Log getLog(String p_strLoggerName) {
        try {
            return (Log) logConstructor.newInstance(p_strLoggerName);
        } catch (Throwable t) {
            throw new RuntimeException("Error creating logger for logger '" + p_strLoggerName + "'.  Cause: " + t, t);
        }
    }


    private static Class<?> getClassByName(String p_strClassName) throws ClassNotFoundException {
        Class<?> clzResult = null;
        try {
            clzResult = Thread.currentThread().getContextClassLoader().loadClass(p_strClassName);
        } catch (Exception e) {
            // Ignore. Failsafe below.
        }
        if (clzResult == null) {
            clzResult = Class.forName(p_strClassName);
        }
        return clzResult;
    }


}