package tofly.modbus_gateway2.log;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.spi.LocationAwareLogger;

public class SLF4JImpl implements Log {
    private static final String strCallerFQCN = SLF4JImpl.class.getName();
    private static final Logger testLogger = LoggerFactory.getLogger(SLF4JImpl.class);

    static {
        // if the logger is not a LocationAwareLogger instance, it can not get correct stack StackTraceElement
        // so ignore this implementation.
        if (!(testLogger instanceof LocationAwareLogger)) {
            throw new UnsupportedOperationException(testLogger.getClass() + " is not a suitable logger");
        }
    }

    private int errorCount;
    private int warnCount;
    private int infoCount;
    private int debugCount;
    private LocationAwareLogger log;

    public SLF4JImpl(LocationAwareLogger p_lalLogger) {
        this.log = p_lalLogger;
    }

    public SLF4JImpl(String p_strLoggerName) {
        this.log = (LocationAwareLogger) LoggerFactory.getLogger(p_strLoggerName);
    }

    @Override
    public boolean isDebugEnabled() {
        return log.isDebugEnabled();
    }

    @Override
    public void error(String p_sMsg, Throwable p_throwable) {
        log.log(null, strCallerFQCN, LocationAwareLogger.ERROR_INT, p_sMsg, null, p_throwable);
        errorCount++;
    }

    @Override
    public void error(String p_sMsg) {
        log.log(null, strCallerFQCN, LocationAwareLogger.ERROR_INT, p_sMsg, null, null);
        errorCount++;
    }

    @Override
    public boolean isInfoEnabled() {
        return log.isInfoEnabled();
    }

    @Override
    public void info(String p_sMsg) {
        infoCount++;
        log.log(null, strCallerFQCN, LocationAwareLogger.INFO_INT, p_sMsg, null, null);
    }

    @Override
    public void debug(String p_sMsg) {
        debugCount++;
        log.log(null, strCallerFQCN, LocationAwareLogger.DEBUG_INT, p_sMsg, null, null);
    }

    @Override
    public void debug(String p_sMsg, Throwable p_throwable) {
        debugCount++;
        log.log(null, strCallerFQCN, LocationAwareLogger.DEBUG_INT, p_sMsg, null, p_throwable);
    }

    @Override
    public boolean isWarnEnabled() {
        return log.isWarnEnabled();
    }

    @Override
    public boolean isErrorEnabled() {
        return log.isErrorEnabled();
    }

    @Override
    public void warn(String p_sMsg) {
        log.log(null, strCallerFQCN, LocationAwareLogger.WARN_INT, p_sMsg, null, null);
        warnCount++;
    }

    @Override
    public void warn(String p_sMsg, Throwable p_throwable) {
        log.log(null, strCallerFQCN, LocationAwareLogger.WARN_INT, p_sMsg, null, p_throwable);
        warnCount++;
    }

    @Override
    public int getErrorCount() {
        return errorCount;
    }

    @Override
    public int getWarnCount() {
        return warnCount;
    }

    @Override
    public int getInfoCount() {
        return infoCount;
    }

    public int getDebugCount() {
        return debugCount;
    }

    @Override
    public void resetStat() {
        errorCount = 0;
        warnCount = 0;
        infoCount = 0;
        debugCount = 0;
    }

}