package tofly.modbus_gateway2.task;

import com.github.zengfr.easymodbus4j.server.ModbusServer4TcpMaster;
import io.netty.channel.Channel;
import io.netty.util.Attribute;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import tofly.modbus_gateway2.config.ModbusServerProperties;

import java.util.Collection;
import java.util.concurrent.TimeUnit;

import static tofly.modbus_gateway2.FirstFrameDecoder.ATTR_KEY_DEVICE_ID;
import static tofly.modbus_gateway2.GwModbusMasterResponseProcessor.ATTR_KEY_REFRESH_LAST_TIME;
import static tofly.modbus_gateway2.HeartbeatFrameDecoder.ATTR_KEY_ALIVE_LAST_TIME;

/***
 * 连接存活检测任务
 */
@Slf4j
@Service
@AllArgsConstructor
public class AliveCheckTask {

    private final ModbusServer4TcpMaster modbusServer4TcpMaster;

    private final ModbusServerProperties properties;


    @Scheduled(fixedRate = 1, timeUnit = TimeUnit.SECONDS)
    public void aliveCheck() {
        Collection<Channel> channels = modbusServer4TcpMaster.getChannels();
        for (Channel channel : channels) {
            if (channel == null || (!channel.isActive()) || !channel.isOpen() || !channel.isWritable()) {
                continue;
            }
            Attribute<Long> attrAliveLastTime = channel.attr(ATTR_KEY_ALIVE_LAST_TIME);
            Attribute<Long> attrRefreshLastTime = channel.attr(ATTR_KEY_REFRESH_LAST_TIME);
            Attribute<String> attr = channel.attr(ATTR_KEY_DEVICE_ID);
            Long aliveLastTime = attrAliveLastTime.get();
            Long refreshLastTime = attrRefreshLastTime.get();//上次数据刷新时间
            String deviceId = attr.get();
            long currentTime = System.currentTimeMillis();
            long seconds = properties.getAliveTimeout().getSeconds();
            if (aliveLastTime != null && currentTime - aliveLastTime > seconds * 1000) {
                close(channel, deviceId, 0);
            }
            if (refreshLastTime != null && currentTime - refreshLastTime > seconds * 1000) {
                close(channel, deviceId, 1);
            }
        }
    }

    private void close(Channel channel, String deviceId, int error) {
        log.warn("连接异常(错误码:{})，关闭连接链路:{}", error, deviceId);
        channel.close();//关闭连接链路
    }

}
