package tofly.modbus_gateway2.transmit;

import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;
import tofly.modbus_gateway2.log.Log;
import tofly.modbus_gateway2.log.LogFactory;
import tofly.modbus_gateway2.util.SystemConst;

import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 报文转发管理（MQTT）
 */
@Component
public class TransmitToMqtt implements ApplicationContextAware {


    /**
     * 日志
     */
    private static final Log log = LogFactory.getLog(TransmitToMqtt.class);

    private static ApplicationContext applicationContext;

    private static ConcurrentHashMap<String, MqttClient> mqttClientCache = new ConcurrentHashMap<>();

    public static void pushData(String p_sMqttClientId, String p_sMqttUsername,
                                String p_sMqttPassword, List<String> p_arrContent) throws MqttException {
        log.info("MQTT客户端Id：" + p_sMqttClientId);
        if (!mqttClientCache.containsKey(p_sMqttClientId)) {
            MqttClient mqttClient = connect(p_sMqttClientId, p_sMqttUsername, p_sMqttPassword);
            mqttClientCache.put(p_sMqttClientId, mqttClient);
        }
        MqttClient mqttClient = mqttClientCache.get(p_sMqttClientId);

        if (!mqttClient.isConnected()) {
            mqttClient.connect(getMqttConnectOptions(p_sMqttUsername, p_sMqttPassword));
        }
        int iQos = 2;
        for (int i = 0; i < p_arrContent.size(); i++) {
            String sContent =  p_arrContent.get(i);
            MqttMessage mm = new MqttMessage(sContent.getBytes());
            mm.setQos(iQos);
            mqttClient.publish("v1/devices/me/telemetry", mm);
        }
        //log.info("Message published");

    }

    public static void saveDeviceDiscovery(String p_sDeviceAddress, String p_sProtocolClassName,
                                           String p_sDeviceId,
                                           String p_sMqttClientId, String p_sMqttUsername,
                                           String p_sMqttPassword, String p_sContent) {
        try {
            DeviceDiscoveryApiService deviceDiscoveryApiService = applicationContext.getBean(DeviceDiscoveryApiService.class);
            DeviceDiscovery deviceDiscovery = new DeviceDiscovery(
                    p_sDeviceId, p_sDeviceAddress,
                    p_sProtocolClassName,
                    p_sContent, p_sMqttClientId,
                    p_sMqttUsername, p_sMqttPassword);
            deviceDiscoveryApiService.saveDeviceDiscovery(deviceDiscovery);
        } catch (Exception e) {
            e.printStackTrace();
            //ignore...
        }
    }

    private static MqttClient connect(String p_sMqttClientId, String p_sMqttUsername, String p_sMqttPassword) throws MqttException {
        String sBroker = "tcp://" + SystemConst.g_platform_ip + ":" + SystemConst.g_mqtt_port;
        MemoryPersistence mp = new MemoryPersistence();
        MqttClient mcClient = new MqttClient(sBroker, p_sMqttClientId, mp);
        mcClient.setCallback(new OnMessageCallback()); // 设置回调
        MqttConnectOptions mco = getMqttConnectOptions(p_sMqttUsername, p_sMqttPassword);
        mcClient.connect(mco);// 建立连接
        log.info("连接MQTT服务成功: " + sBroker);
        return mcClient;
    }

    private static MqttConnectOptions getMqttConnectOptions(String p_sMqttUsername, String p_sMqttPassword) {
        // MQTT 连接选项
        MqttConnectOptions mco = new MqttConnectOptions();
        mco.setUserName(p_sMqttUsername);
        mco.setPassword(p_sMqttPassword.toCharArray());
        mco.setAutomaticReconnect(true);//自动重连
        // 保留会话
        mco.setCleanSession(true);
        return mco;
    }

    public static void disconnect(String p_sDeviceId) {
        mqttClientCache.computeIfPresent(p_sDeviceId, (s, mqttClient) -> {
            try {
                mqttClient.disconnect();
                return mqttClient;
            } catch (MqttException e) {
                throw new RuntimeException(e);
            }
        });
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext2) throws BeansException {
        applicationContext = applicationContext2;
    }

    public static class OnMessageCallback implements MqttCallback {
        public void connectionLost(Throwable p_t) {
            log.info("连接断开");
        }

        public void messageArrived(String p_sTopic, MqttMessage p_mm) throws Exception {
            log.info("接收消息主题:" + p_sTopic);
            log.info("接收消息Qos:" + p_mm.getQos());
            log.info("接收消息内容:" + new String(p_mm.getPayload()));
        }

        public void deliveryComplete(IMqttDeliveryToken p_mdtToken) {
            log.info("消息转发完成:" + p_mdtToken.isComplete());
        }
    }
}
